%r=filmFieldTE(r, s, w, P)
%-------------------------
%
%Computes the electromagnetic field through an array of slits in
%a metal layer. See 'filmTransmissionTE' for a description of the
%model geometry.
%
%The incident plane wave has an intensity of 1W/m^2.
%
%Input:
% r      Structure (see 'filmTransmissionTE')
% s      Drawing dimensions [m] {[px py h+3*l]}
% w      Image dimensions [pixel] {[256 1 256]}
% P      Compute up to |kx|=P*k {all}
%
%Output:
% r      Structure completed with
%  .I       Intensity [W/m^2]
%  .P       Used limit for kx
%  .S       Drawing dimensions [m]
%  .W       Image dimensions [pixel]
%  .x       Computed points [m]
%  .y
%  .z
%  .Ey      Electric field [V/m]
%  .Hx      Magnetic field [A/m]
%  .Hz
%  .gr      kx/k0 of the reflection
%  .gt      kx/k0 of the transmission
%  .Fr      Reflection spectrum [W/m^2]
%  .Ft      Transmission spectrum [W/m^2]
%
function r=filmFieldTE(r, s, w, P)
if ~nargin | isempty(r) | ~isstruct(r)
   help filmFieldTE
   return;
end
if nargin < 2 | length(s) ~= 3
   s=[r.px r.py r.h+3*r.l];
else
   s=abs(s);
end
if nargin < 3 | length(w) ~= 3
   w=[256 1 256];
else
   w=ceil(abs(w));
   w(2)=1;
end
if nargin < 4 | length(P) ~= 1
   P=[];
else
   P=abs(P);
end
statusbar('Field computation ...');
if ~isfield(r,'um')
   r=filmTransmissionTE(r,P);
end
%
%Sampling points
%
t=s./w;
x=((1:w(1))-w(1)/2)*t(1);
if real(r.e4) < 0
   z=((1:w(3))-0.9*w(3))*t(3) + r.h/2;
else
   z=((1:w(3))-0.3*w(3))*t(3) + r.h/2;
end
ki=i*r.k0;
%
%Field allocations
%
r.I=zeros(w);
r.P=P;
r.S=s;
r.W=w;
r.x=x;
r.y=0;
r.z=z;
r.Ey=r.I;
r.Hx=r.I;
r.Hz=r.I;
%
%Regions and slit positions
%
% Layer at       z(a:b)
% Slits at       x(c)
%
a=find(z >= 0);
a=a(1);
b=find(z <= r.h);
b=b(end);
c=[];
n=ceil(s(1)/r.px);
for j=-n:n
   c=[c find(abs(x-j*r.px) < r.wx/2)];
end
%
%Region 1:   Input
%
% Ey = exp(i*k0*(g0*x + b1*z)) + sum{Fn*exp(i*k0*(gn*x - bn*z))}
% Hx = -b*sqrt(e0/u0)*Ey
% Hz = g*sqrt(e0/u0)*Ey
%
kz=-ki*r.b1;
n1=sqrt(r.e1);
s=exp(ki*r.g0*x);
t=exp(ki*r.gn.'*x);
Fn=r.Cm.*r.Rm*r.um.'*exp(ki*r.xm*(r.g0-r.gn))/length(r.xm);
r.gr=r.gn;
r.Fr=real(Fn.*conj(Fn));
r.I(:,:,1:a-1)=1;
for j=1:a-1
   u=Fn.*exp(kz*z(j));
   v=s*exp(ki*r.b0*z(j));
   r.Ey(:,1,j)=u*t + v;
   r.Hx(:,1,j)=r.b1.*u*t - r.b0*v;
   r.Hz(:,1,j)=r.gn.*u*t + r.g0*v;
   if isempty(statusbar(j/w(3),f))
      return;
   end
end
%
%Region 2/3: Layer
%
% Ey = sum{(Am*exp(i*k0*bm*z) + Bm*exp(-i*k0*bm*z))*Cm*um}*exp(i*k0*g0*x)
%
n=length(r.xm);
t=round(r.S(1)/r.px/2);
um=repmat((-t:t)*r.px,size(r.xm));
dm=interp1(repmat(r.xm,1+2*t,1)+um(:),repmat((r.um([2:n 1],:)-r.um([n 1:n-1],:))/(r.xm(3)-r.xm(1)),1+2*t,1),x,'spline').';
um=interp1(repmat(r.xm,1+2*t,1)+um(:),repmat(r.um,1+2*t,1),x,'spline').';
Am=r.Am.*r.Cm;
Bm=r.Bm.*r.Cm;
kz=ki*r.bm;
n2=sqrt(r.e2);
n3=sqrt(r.e3);
r.I(:,:,a:b)=n2/n1;
r.I(c,:,a:b)=n3/n1;
for j=a:b
   t=exp(kz*z(j));
   u=Am.*t;
   v=Bm./t;
   r.Ey(:,1,j)=(u+v)*um.*s;
   r.Hx(:,1,j)=r.bm.*(v-u)*um.*s;
   r.Hz(:,1,j)=(r.g0*(u+v)*um - i/r.k0*(u+v)*dm).*s;
   if isempty(statusbar(j/w(3),f))
      return;
   end
end
%
%Region 4:   Output
%
% Ey = sum{Fn*exp(i*k0*(gn*x + bn*(z-h)))}
%
kz=ki*r.b4;
n4=sqrt(r.e4);
t=exp(ki*r.gn.'*x);
Fn=r.Cm.*r.Tm*r.um.'*exp(ki*r.xm*(r.g0-r.gn))/length(r.xm);
r.gt=r.gn;
r.Ft=real(n4/n1*Fn.*conj(Fn));
r.I(:,:,1+b:w(3))=n4/n1;
for j=1+b:w(3)
   u=Fn.*exp(kz*(z(j)-r.h));
   r.Ey(:,1,j)=u*t;
   r.Hx(:,1,j)=-r.b4.*u*t;
   r.Hz(:,1,j)=r.gn.*u*t;
   if isempty(statusbar(j/w(3),f))
      return;
   end
end
%
%Intensity & normalization for 1W/m^2 input
%
% I = 1/2*real(E*conj(E)*sqrt(e0*er/u0))
% s = Ei(I = 1W/m^2)
%
e0=8.85418782e-12; % [As/Vm]
u0=1.25663706e-6;  % [Vs/Am]
s=sqrt(2/n1*sqrt(u0/e0));
r.I=single(real(r.I.*r.Ey.*conj(r.Ey)));
r.Ey=single(r.Ey*s);
s=sqrt(2/n1*sqrt(e0/u0));
r.Hx=single(r.Hx*s);
r.Hz=single(r.Hz*s);
delete(f);
