%r=filmFieldTM(r, s, w, P)
%-------------------------
%
%Computes the electromagnetic field through an array of slits in
%a metal layer. See 'filmTransmissionTM' for a description of the
%model geometry.
%
%The incident plane wave has an intensity of 1W/m^2.
%
%Input:
% r      Structure (see 'filmTransmissionTM')
% s      Drawing dimensions [m] {[px py h+3*l]}
% w      Image dimensions [pixel] {[256 1 256]}
% P      Compute up to |kx|=P*k {all}
%
%Output:
% r      Structure completed with
%  .I       Intensity [W/m^2]
%  .P       Used limit for kx
%  .S       Drawing dimensions [m]
%  .W       Image dimensions [pixel]
%  .x       Computed points [m]
%  .y
%  .z
%  .Hy      Magnetic field [A/m]
%  .Ex      Electric field [V/m]
%  .Ez      (not yet implemented)
%  .gr      kx/k0 of the reflection
%  .gt      kx/k0 of the transmission
%  .Fr      Reflection spectrum [W/m^2]
%  .Ft      Transmission spectrum [W/m^2]
%
function r=filmFieldTM(r, s, w, P)
if ~nargin | isempty(r) | ~isstruct(r)
   help filmFieldTM
   return;
end
if nargin < 2 | length(s) ~= 3
   s=[r.px r.py r.h+3*r.l];
else
   s=abs(s);
end
if nargin < 3 | length(w) ~= 3
   w=[256 1 256];
else
   w=ceil(abs(w));
   w(2)=1;
end
if nargin < 4 | length(P) ~= 1
   P=[];
else
   P=abs(P);
end
statusbar('Field computation ...');
if ~isfield(r,'um')
   r=filmTransmissionTM(r,P);
end
%
%Sampling points
%
t=s./w;
x=((1:w(1))-w(1)/2)*t(1);
if real(r.e4) < 0
   z=((1:w(3))-0.9*w(3))*t(3) + r.h/2;
else
   z=((1:w(3))-0.3*w(3))*t(3) + r.h/2;
end
ki=i*r.k0;
%
%Field allocations
%
r.I=zeros(w);
r.P=P;
r.S=s;
r.W=w;
r.x=x;
r.y=0;
r.z=z;
r.Hy=r.I;
%
%Regions and slit positions
%
% Layer at       z(a:b)
% Slits at       x(c)
%
a=find(z >= 0);
a=a(1);
b=find(z <= r.h);
b=b(length(b));
c=[];
n=ceil(s(1)/r.px);
for j=-n:n
   c=[c find(abs(x-j*r.px) < r.wx/2)];
end
%
%Region 1:   Input
%
% Hy = exp(i*k0*(g0*x + b1*z)) + sum{Fn*exp(i*k0*(gn*x - bn*z))}
%
kz=-ki*r.b1;
n1=sqrt(r.e1);
s=exp(ki*r.g0*x);
t=exp(ki*r.gn.'*x);
Fn=((r.Cm.*r.Rm)*r.um.')*exp(ki*r.xm*(r.g0-r.gn))/length(r.xm);
r.gr=r.gn;
r.Fr=real(Fn.*conj(Fn));
r.I(:,:,1:a-1)=1;
for j=1:a-1
   r.Hy(:,1,j)=s*exp(ki*r.b0*z(j)) + (Fn.*exp(kz*z(j)))*t;
   if isempty(statusbar(j/w(3),f))
      return;
   end
end
%
%Region 2/3: Layer
%
% Hy = sum{(Am*exp(i*k0*bm*z) + Bm*exp(-i*k0*bm*z))*Cm*um}*exp(i*k0*g0*x)
%
t=round(r.S(1)/r.px/2);
um=repmat((-t:t)*r.px,size(r.xm));
um=interp1(repmat(r.xm,1+2*t,1)+um(:),repmat(r.um,1+2*t,1),x,'spline').';
Am=r.Am.*r.Cm;
Bm=r.Bm.*r.Cm;
kz=ki*r.bm;
n2=sqrt(r.e2);
n3=sqrt(r.e3);
r.I(:,:,a:b)=n1/n2;
r.I(c,:,a:b)=n1/n3;
for j=a:b
   t=exp(kz*z(j));
   r.Hy(:,1,j)=((Am.*t + Bm.*conj(t))*um).*s;   % ???
%   r.Hy(:,1,j)=(Am.*t + Bm./t)*um.*s;
   if isempty(statusbar(j/w(3),f))
      return;
   end
end
%
%Region 4:   Output
%
% Hy = sum{Fn*exp(i*k0*(gn*x + bn*(z-h)))}
%
kz=ki*r.b4;
n4=sqrt(r.e4);
t=exp(ki*r.gn.'*x);
Fn=((r.Cm.*r.Tm)*r.um.')*exp(ki*r.xm*(r.g0-r.gn))/length(r.xm);
r.gt=r.gn;
r.Ft=real(n1/n4*Fn.*conj(Fn));
r.I(:,:,1+b:w(3))=n1/n4;
for j=1+b:w(3)
   r.Hy(:,1,j)=(Fn.*exp(kz*(z(j)-r.h)))*t;
   if isempty(statusbar(j/w(3),f))
      return;
   end
end
%
%Intensity & normalization for 1W/m^2 input
%
% I = 1/2*real(H*conj(H)*sqrt(u0/e0*er))
% s = Hi(I = 1W/m^2)
%
e0=8.85418782e-12; % [As/Vm]
u0=1.25663706e-6;  % [Vs/Am]
s=sqrt(2*n1*sqrt(e0/u0));
r.I=single(real(r.I.*r.Hy.*conj(r.Hy)));
r.Hy=single(r.Hy*s);
delete(f);
