%r=filmTransmissionTM(r, P)
%--------------------------
%
%Computes the electromagnetic field produced by a single slit
%as shown below when lit with a plane TM polarised wave with
%the xz plane as incidence plane.
%
%               w
%     1      -----      1
%               y
%------------ --o-- ------------       -      ---> x
%                                    
%     2        3        2          h 
%                                    
%-------------------------------       -
%
%               4
%
%-------------------------------
%
%               
%               
%               v
%               z
%
%
%The incident light impinges through the dielectricum 1 onto
%the metal layer 2 and the slit 3. A part is diffracted back
%into the region 1, another part is transmitted through the
%slit into the substrate 4.
%
%The incident light has an magnetic field Hy along the y-axis.
%Hence, its electric field E=(Ex,0,Ez) lies in the xz-plane.
%At the boundaries, the surfacial components of the magnetic
%and of the electric field have to be continuos.
%
%Input:
% r      Structure
%  .i       Incidence angle []
%              0 = z-axis
%             90 = x-axis
%  .l       Wavelength in free space [m]
%  .h       Thickness of the metal layer [m]
%  .wx      Width of the slits [m]
%  .px      Distance between the slits [m]
%  .e1      Relative dielectric constants
%  .e2
%  .e3
%  .e4
%
% P      Compute up to |kx|=P*k {10}
%
%Output:
% r      Structure completed with
%  .k0      Wavevector in free space [rad/m]
%  .g0      kx/k0 of the incident wave
%  .gn      kx/k0 of the Rayleigh modes
%  .b1      Propagation constants kz/k0
%  .b4
%  .bm
%  .em      Effectife dielectric constants
%  .um      Bloch modes of the slit array
%  .xm      Sampling points
%  .Cm      Coupling coefficients
%  .Am      Coefficients of the forward propagating wave
%  .Bm      Coefficients of the backward propagating wave
%  .Rm      Reflection coefficients at the slit array
%  .Tm      Transmission coefficients through the slit array
%
function r=filmTransmissionTM(r, P)
if nargin < 2 | isempty(P)
   P=10;
else
   P=max(1,P);
end
%
% Wavevector and propagation constants
%
t=r.l/r.px;
s=r.l/r.wx/2;
r.k0=2*pi/r.l;
r.g0=real(sqrt(r.e1)*sin(r.i*pi/180));
r.gn=r.g0+(-floor((r.g0+P)/t):floor((P-r.g0)/t))*t;
r.b0=zSpeed(r.e1,r.g0);
r.b1=zSpeed(r.e1,r.gn);
r.b4=zSpeed(r.e4,r.gn);
%
% Transversal Bloch modes
%
N=max(63,10*P/t);
[u,e,x]=filmModesTM(r.e2,r.e3,r.wx,r.px,r.g0,r.k0,N,N/5);
r.bm=zSpeed(e,r.g0);
r.em=e;
r.um=u;
r.xm=x;
%
% Coupling coefficients to the slit modes
%
r.Cm=conj(sum(u,1));
%
% Fresnel reflection and transmission through the slit
%
[R,T,A,B]=layerRTtm(r.e1,r.em,r.e4,r.g0,r.k0,r.h);
r.Am=A;
r.Bm=B;
r.Rm=R;
r.Tm=T;
%
%Print mean coupling error
%
t=r.um*r.Cm.'-1;
fprintf('Total relative intensity mismatch Ptot = %g%%\n',100*mean(real(t.*conj(t))));
