%r=lensField(r, s, w, P)
%-----------------------
%
%Computes the electromagnetic field behind an array of cylindrical
%lenses in an opaque screen.
%
%The incident plane TE wave has an intensity of 1W/m^2.
%
%Input:
% r      Structure
%  .i       Incidence angle []
%  .f       Focal length [m]
%  .l       Wavelength in free space [m]
%  .px      Array period [m]
%  .wx      Lens width [m]
%  .e1      Relative dielectric constant before the lens
%  .e3      Relative dielectric constant the lens is designed for
%  .e4      Relative dielectric constant behind the lens
% s      Drawing dimensions [m] {[px py 2*f]}
% w      Image dimensions [pixel] {[256 1 256]}
% P      Compute up to |kx|=P*k {NA}
%
%Output:
% r      Structure completed with
%  .I       Intensity [W/m^2]
%  .P       Used limit for kx
%  .S       Drawing dimensions [m]
%  .W       Image dimensions [pixel]
%  .x       Computed points [m]
%  .y
%  .z
%  .Ey      Electric field [V/m]
%  .Hx      Magnetic field [A/m]
%  .Hz
%  .gt      kx/k0 of the transmission
%  .Ft      Transmission spectrum [W/m^2]
%
function r=lensField(r, s, w, P)
if ~nargin | isempty(r) | ~isstruct(r)
   help lensField
   return;
end
if nargin < 2 | length(s) ~= 3
   s=[r.px r.py 2*r.f];
else
   s=abs(s);
end
if nargin < 3 | length(w) ~= 3
   w=[256 1 256];
else
   w=ceil(abs(w));
   w(2)=1;
end
if nargin < 4 | length(P) ~= 1
   P=real(sqrt(r.e4))*r.wx/sqrt(r.wx^2+4*r.f^2);
else
   P=abs(P);
end
f=statusbar('Lens field ...');
%
% Wavevector and propagation constants
%
t=r.l/r.px;
r.k0=2*pi/r.l;
r.g0=real(sqrt(r.e1)*sin(r.i*pi/180));
r.gn=r.g0+(-floor((r.g0+P)/t):floor((P-r.g0)/t))*t;
r.b0=zSpeed(r.e1,r.g0);
r.b4=zSpeed(r.e4,r.gn);
%
%Sampling points
%
t=s./w;
x=((1:w(1))-w(1)/2)*t(1);
z=((1:w(3))-w(3)/2)*t(3)+r.f;
if z(1) < 0
   z=z-z(1);
end
ki=i*r.k0;
%
%Field allocations
%
r.I=zeros(w);
r.P=P;
r.S=s;
r.W=w;
r.x=x;
r.y=0;
r.z=z;
r.Ey=r.I;
r.Hx=r.I;
r.Hz=r.I;
%
%Region 4:   Output
%
% Ey = sum{Fn*exp(i*k0*(gn*x + bn*z))}
%
kz=ki*r.b4;
n1=sqrt(r.e1);
n3=sqrt(r.e3);
n4=sqrt(r.e4);
s=exp(ki*r.g0*x);
t=exp(ki*r.gn.'*x);
N=max(63,10*r.px*r.P/r.l);
r.xm=(-floor(N/2):floor(N/2))'*r.px/N;
r.um=exp(ki*real(n3)*(r.f-sqrt(r.f^2+r.xm.^2))).*(abs(r.xm) < r.wx/2);
Fn=r.um.'*exp(ki*r.xm*(r.g0-r.gn))/length(r.xm);
r.gt=r.gn;
r.Ft=real(n4/n1*Fn.*conj(Fn));
r.I(:,:,:)=n4/n1;
for j=1:w(3)
   u=Fn.*exp(kz*z(j));
   r.Ey(:,1,j)=u*t;
   r.Hx(:,1,j)=-r.b4.*u*t;
   r.Hz(:,1,j)=r.gn.*u*t;
   if isempty(statusbar(j/w(3),f))
      return;
   end
end
%
%Intensity & normalization for 1W/m^2 input
%
% I = 1/2*real(E*conj(E)*sqrt(e0*er/u0))
% s = Ei(I = 1W/m^2)
%
e0=8.85418782e-12; % [As/Vm]
u0=1.25663706e-6;  % [Vs/Am]
s=sqrt(2/n1*sqrt(u0/e0));
r.I=single(real(r.I.*r.Ey.*conj(r.Ey)));
r.Ey=single(r.Ey*s);
s=sqrt(2/n1*sqrt(e0/u0));
r.Hx=single(r.Hx*s);
r.Hz=single(r.Hz*s);
delete(f);
