%[u,e,x]=filmModesTM(e2,e3,wx,px,g0,k0,N,M)
%------------------------------------------
%
%Find the TM eigenmodes of an array of slits in a metal
%film by means of solving an eigenvalue problem.
%
%Input:
% e2/e3  Dielectric constant of the film and the slits
% wx     Width of the slits [m]
% px     Slit array period [m]
% g0     Incidence angle kx/k0
% k0     Wavevector in free space [rad/m]
% N      Number of sampling points {127}
% M      Number of calculated eigenmodes {all}
%
%Output:
% u      A matrix of Bloch functions. The magnetic field of
%        the eigenmode m is Hm(x) = u(:,m)*exp(i*k0*g0*x)
% e      Effective dielectric constant
% x      Sampling points
%
function [u,e,x]=filmModesTM(e2,e3,wx,px,g0,k0,N,M,t)
if nargin < 6
   help filmModesTM
   return;
end
if nargin < 7 | isempty(N)
   N=127;
else
   N=round(N);
end
if nargin < 8
   M=1;
end
s=px/N;
e=repmat(e2,N,1);
x=((s-px)/2:s:px/2).';
e(abs(x) < wx/2)=e3;
%
%Sparse matrix definition
%
a=e([2:N 1]);
b=e([N 1:N-1]);
c=(a+b)/2;
d=(a-b)./(2*s*c);
a=c*k0^2 - i*k0*g0*d;
b=(2i*k0*g0 - d)/(2*s);
c=1/s^2;
u=spdiags([c-[b(2:N);NaN] a-2*c [NaN;b(1:N-1)]+c],[-1 0 1],N,N);
u(1,N)=c-b(1);
u(N,1)=b(N)+c;
%
%Eigenvalue computation
%
L=1024;
if M < N & (N > L | nargin > 7)
   s.disp=0;
   s.maxit=1000;
   M=min(max([L+round(sqrt(N-L)) round(M)]),N-2);
   [u,e,s]=eigs(inv(u),M,'lm',s);
   if s
      fprintf('Not all eigenvalues converged.\n');
   end
   e=1./(diag(e)*k0^2).';
else
   [u,e]=eig(full(u));
   e=diag(e).'/k0^2;
end
