%[gn,Fn]=rayleighCte(C0,g0,Cm,gm,k0,wx,px,P)
%-------------------------------------------
%
%Computes the field difference at the slit boundary and
%returns the coefficients of the Rayleigh expansion.
%
%Input:
% C0     Coefficient at the metal layer
% g0     kx/k0 of the incident wave
% Cm     Modal coefficients at the slit
% gm     kx/k0 of the TE hole modes
% k0     Wavevector in free space [rad/m]
% wx     Width of the slits [m]
% px     Distance between the slits [m]
% P      Compute up to |kx|=P*k0 {all}
%
%Output:
% gn     kx/k0 of the diffracted waves
% Fn     Rayleigh coefficients
%
function [gn,Fn]=rayleighCte(C0,g0,Cm,gm,k0,wx,px,P)
%
%Field difference
%
% dF = perturbation
%
t=max(50,5*length(gm));
s=wx/t/2;
x=s*(-t:t);
t=2*t*px/wx;
dF=Cm*sin(k0*gm'*(x+wx/2))-C0*exp(i*k0*g0*x);
%
%Avoid aliasing (Shannon)
%
P=min([P pi/k0/s]);
%
%Rayleigh expansion
%
% gn = g0 + n*px/l    for all |gn| < P
%
s=2*pi/k0/px;
gn=g0+(-floor((P+g0)/s):floor((P-g0)/s))*s;
Fn=dF*exp(-i*k0*x'*gn)/t;
