%r=slitFieldTE(r, s, w, P, O)
%----------------------------
%
%Computes the electromagnetic field through an array of
%slits in a metal layer. See 'slitTransmissionTE' for a
%description of the model geometry.
%
%The incident plane wave has an intensity of 1W/m^2.
%
%Input:
% r      Structure (see 'slitTransmissionTE')
% s      Drawing dimensions [m] {[px py h+3*l]}
% w      Image dimensions [pixel] {[256 1 256]}
% P      Compute up to |kx|=P*k0 {all}
% O      Chooses the plane wave expansion method
%        {'Rayleigh'} or 'Fourier'
%
%Output:
% r      Structure completed with
%  .I       Intensity [W/m^2]
%  .P       Used limit for kx
%  .S       Drawing dimensions [m]
%  .W       Image dimensions [pixel]
%  .x       Computed points [m]
%  .y
%  .z
%  .Ey      Electric field [V/m]
%  .Hx      Magnetic field [A/m]
%  .Hz
%  .gr      kx/k0 of the reflection
%  .gt      kx/k0 of the transmission
%  .Fr      Reflection spectrum [W/m^2]
%  .Ft      Transmission spectrum [W/m^2]
%
function r=slitFieldTE(r, s, w, P, O)
if ~nargin | isempty(r) | ~isstruct(r)
   help slitFieldTE
   return;
end
if nargin < 2 | length(s) ~= 3
   s=[r.px r.py r.h+3*r.l];
else
   s=abs(s);
end
if nargin < 3 | length(w) ~= 3
   w=[256 1 256];
else
   w=ceil(abs(w));
   w(2)=1;
end
if nargin < 4 | length(P) ~= 1
   P=[];
else
   P=abs(P);
end
if nargin < 5
   O='';
end
switch lower(O)
case 'fourier'
   O=1;
otherwise
   O=0;
end
statusbar('Field computation ...');
if ~isfield(r,'A3')
   r=slitTransmissionTE(r,P);
end
%
%Sampling points
%
t=s./w;
x=((1:w(1))-w(1)/2)*t(1);
if real(r.e4) < 0
   z=((1:w(3))-0.9*w(3))*t(3) + r.h/2;
else
   z=((1:w(3))-0.3*w(3))*t(3) + r.h/2;
end
ki=i*r.k0;
%
%Field allocations
%
r.I=zeros(w);
r.P=abs(P);
r.S=s;
r.W=w;
r.x=x;
r.y=0;
r.z=z;
r.Ey=r.I;
r.Hx=r.I;
r.Hz=r.I;
%
%Regions and slit positions
%
% Metal layer at z(a)...z(b)
% Holes at       x(c)
% Local x        d
% Hole phasors   e
%
a=find(z >= 0);
a=a(1);
b=find(z <= r.h);
b=b(length(b));
c=[];
d=[];
e=[];
n=ceil(s(1)/r.px);
for j=-n:n
   t=find(abs(x+j*r.px) < r.wx/2);
   c=[c t];
   d=[d x(t)+r.wx/2+j*r.px];
   e=[e repmat(exp(-j*ki*r.g0*r.px),size(t))];
end
%
%Region 1:   Input
%
% Ey = exp(i*k0*(g0*x + b1*z)) + R2*exp(i*k0*(g0*x - b1*z))
%    + sum(Fn*exp(i*k0*(gn*x - bn*z)))
% Hx = -b*sqrt(e0/u0)*Ey
% Hz = g*sqrt(e0/u0)*Ey
%
s=exp(ki*r.g0*x);
n1=sqrt(r.e1);
if O
   [gn,Fn]=fourierCte(1+r.R2,r.g0,(1+r.R3).*r.Cm,r.g3,r.k0,r.wx,r.px,r.P);
   u=exp(ki*gn'*(x+r.wx/2));
else
   [gn,Fn]=rayleighCte(1+r.R2,r.g0,(1+r.R3).*r.Cm,r.g3,r.k0,r.wx,r.px,r.P);
   u=exp(ki*gn'*x);
end
r.gr=gn;
r.Fr=real(Fn.*conj(Fn));
bn=zSpeed(r.e1,gn);
r.I(:,:,1:a-1)=1;
for j=1:a-1
   l=ki*z(j)*conj(r.b1);
   t=r.R2*exp(conj(l));
   l=exp(l);
   v=exp(-ki*z(j)*bn);
   r.Ey(:,1,j)=(l+t)*s + (Fn.*v)*u;
   r.Hx(:,1,j)=r.b1*(t-l)*s + (Fn.*bn.*v)*u;
   r.Hz(:,1,j)=r.g0*(l+t)*s + (Fn.*gn.*v)*u;
   if isempty(statusbar(j/w(3),f))
      return;
   end
end
%
%Region 2:   Layer
%
% Ey = A2*exp(i*k0*(g0*x + b2*z)) + B2*exp(i*k0*(g0*x - b2*z))
%
%Region 3:   Slits
%
% Ey = sum((A3*exp(i*k0*b3*z) + B3*exp(-i*k0*b3*z))*sin(k0*g3*x));
%
n2=sqrt(r.e2);
n3=sqrt(r.e3);
r.I(:,:,a:b)=n2/n1;
r.I(c,:,a:b)=n3/n1;
u=exp(ki*r.g3'*d);
for j=a:b
   t=exp(ki*z(j)*r.b2);
   r.Ey(:,1,j)=(r.B2/t+r.A2*t)*s;
   r.Hx(:,1,j)=r.b2*(r.B2/t-r.A2*t)*s;
   r.Hz(:,1,j)=r.g0*(r.B2/t+r.A2*t)*s;
   t=exp(ki*z(j)*r.b3);
   v=(r.B3./t+r.A3.*t).*r.Cm;
   r.Ey(c,:,j)=v*imag(u).*e;
   r.Hx(c,:,j)=(r.b3.*(r.B3./t-r.A3.*t).*r.Cm)*imag(u).*e;
   r.Hz(c,:,j)=-i*(r.g3.*v)*real(u).*e;
   if isempty(statusbar(j/w(3),f))
      return;
   end
end
%
%Region 4:   Output
%
% Ey = T2*exp(i*k0*(g0*x + b4*z))
%    + sum(Fn*exp(i*k0*(gn*x + bn*(z-h))))
%
n4=sqrt(r.e4);
if O
   [gn,Fn]=fourierCte(r.T2,r.g0,r.T3.*r.Cm,r.g3,r.k0,r.wx,r.px,r.P);
   u=exp(ki*gn'*(x+r.wx/2));
else
   [gn,Fn]=rayleighCte(r.T2,r.g0,r.T3.*r.Cm,r.g3,r.k0,r.wx,r.px,r.P);
   u=exp(ki*gn'*x);
end
r.gt=gn;
r.Ft=real(n4/n1*Fn.*conj(Fn));
bn=zSpeed(r.e4,gn);
r.I(:,:,1+b:w(3))=n4/n1;
for j=1+b:w(3)
   t=exp(ki*(z(j)-r.h)*r.b4);
   v=exp(ki*(z(j)-r.h)*bn);
   r.Ey(:,1,j)=r.T2*t*s + (Fn.*v)*u;
   r.Hx(:,1,j)=-r.b4*r.T2*t*s - (Fn.*bn.*v)*u;
   r.Hz(:,1,j)=r.g0*r.T2*t*s + (Fn.*gn.*v)*u;
   if isempty(statusbar(j/w(3),f))
      return;
   end
end
%
%Intensity & normalization for 1W/m^2 input
%
% I = 1/2*real(E*conj(E)*sqrt(e0*er/u0))
% s = Ei(I = 1W/m^2)
%
e0=8.85418782e-12; % [As/Vm]
u0=1.25663706e-6;  % [Vs/Am]
s=sqrt(2/n1*sqrt(u0/e0));
r.I=single(real(r.I.*r.Ey.*conj(r.Ey)));
r.Ey=single(r.Ey*s);
s=sqrt(2/n1*sqrt(e0/u0));
r.Hx=single(r.Hx*s);
r.Hz=single(r.Hz*s);
delete(f);
