%r=slitFieldTM(r, w, w, P, O)
%----------------------------
%
%Computes the electromagnetic field through an array of
%slits in a metal layer. See 'slitTransmissionTM' for a
%description of the model geometry.
%
%The incident plane wave has an intensity of 1W/m^2.
%
%Input:
% r      Structure (see 'slitTransmissionTM')
% s      Drawing dimensions [m] {[px py h+3*l]}
% w      Image dimensions [pixel] {[256 1 256]}
% P      Compute up to |kx|=P*k0 {all}
% O      Chooses the plane wave expansion method
%        {'Rayleigh'} or 'Fourier'
%
%Output:
% r      Structure completed with
%  .I       Intensity [W/m^2]
%  .P       Used limit for kx
%  .S       Drawing dimensions [m]
%  .W       Image dimensions [pixel]
%  .x       Computed points [m]
%  .y
%  .z
%  .Ex      Electric field [V/m]
%  .Ez
%  .Hy      Magnetic field [A/m]
%  .gr      kx/k0 of the reflection
%  .gt      kx/k0 of the transmission
%  .Fr      Reflection spectrum [W/m^2]
%  .Ft      Transmission spectrum [W/m^2]
%
function r=slitFieldTM(r, s, w, P, O)
if ~nargin | isempty(r) | ~isstruct(r)
   help slitFieldTM
   return;
end
if nargin < 2 | length(s) ~= 3
   s=[r.px r.py r.h+3*r.l];
else
   s=abs(s);
end
if nargin < 3 | length(w) ~= 3
   w=[256 1 256];
else
   w=ceil(abs(w));
   w(2)=1;
end
if nargin < 4 | length(P) ~= 1
   P=[];
else
   P=abs(P);
end
if nargin < 5
   O='';
end
switch lower(O)
case 'fourier'
   O=1;
otherwise
   O=0;
end
statusbar('Field computation ...');
if ~isfield(r,'A3')
   r=slitTransmissionTM(r,P);
end
%
%Sampling points
%
t=s./w;
x=((1:w(1))-w(1)/2)*t(1);
if real(r.e4) < 0
   z=((1:w(3))-0.9*w(3))*t(3) + r.h/2;
else
   z=((1:w(3))-0.3*w(3))*t(3) + r.h/2;
end
ki=i*r.k0;
%
%Field allocations
%
r.I=zeros(w);
r.P=abs(P);
r.S=s;
r.W=w;
r.x=x;
r.y=0;
r.z=z;
r.Ex=r.I;
r.Ez=r.I;
r.Hy=r.I;
%
%Regions and slit positions
%
% Metal layer at z(a)...z(b)
% Holes at       x(c)
% Local x        d
% Hole phasors   e
%
a=find(z >= 0);
a=a(1);
b=find(z <= r.h);
b=b(length(b));
c=[];
d=[];
e=[];
n=ceil(s(1)/r.px);
for j=-n:n
   t=find(abs(x+j*r.px) < r.wx/2);
   c=[c t];
   d=[d x(t)+r.wx/2+j*r.px];
   e=[e repmat(exp(-j*ki*r.g0*r.px),size(t))];
end
%
%Region 1:   Input
%
% Hy = exp(i*k0*(g0*x + b1*z)) + R2*exp(i*k0*(g0*x - b1*z))
%    + sum(Fn*exp(i*k0*(gn*x - bn*z)))
% Ex = b*sqrt(u0/e0)/er*Hy
% Ez = -g*sqrt(u0/e0)/er*Hy
%
s=exp(ki*r.g0*x);
n1=sqrt(r.e1);
if O
   [gn,Fn]=fourierCtm(1+r.R2,r.g0,(1+r.R3).*r.Cm,r.g3,r.k0,r.wx,r.px,r.P);
   u=exp(ki*gn'*(x+r.wx/2));
else
   [gn,Fn]=rayleighCtm(1+r.R2,r.g0,(1+r.R3).*r.Cm,r.g3,r.k0,r.wx,r.px,r.P);
   u=exp(ki*gn'*x);
end
r.gr=gn;
t=abs(gn-r.g0);
t=find(t == min(t));
r.Fr=real(Fn.*conj(Fn));
r.Fr(t)=real((Fn(t)+r.R2)*conj(Fn(t)+r.R2));
bn=zSpeed(r.e1,gn);
r.I(:,:,1:a-1)=1;
for j=1:a-1
   l=ki*z(j)*conj(r.b1);
   t=r.R2*exp(conj(l));
   l=exp(l);
   v=exp(-ki*z(j)*bn);
   r.Hy(:,1,j)=(l+t)*s + (Fn.*v)*u;
   r.Ex(:,1,j)=(r.b1*(l-t)*s - (Fn.*bn.*v)*u)/r.e1;
   r.Ez(:,1,j)=(-r.g0*(l+t)*s - (Fn.*gn.*v)*u)/r.e1;
   if isempty(statusbar(j/w(3),f))
      return;
   end
end
%
%Region 2:   Layer
%
% Hy = A2*exp(i*k0*(g0*x + b2*z)) + B2*exp(i*k0*(g0*x - b2*z))
%
%Region 3:   Slits
%
% Hy = sum((A3*exp(i*k0*b3*z) + B3*exp(-i*k0*b3*z))*cos(k0*g3*x));
%
n2=sqrt(r.e2);
n3=sqrt(r.e3);
r.I(:,:,a:b)=n1/n2;
r.I(c,:,a:b)=n1/n3;
u=exp(ki*r.g3'*d);
for j=a:b
   t=exp(ki*z(j)*r.b2);
   r.Hy(:,1,j)=(r.A2*t+r.B2/t)*s;
   r.Ex(:,1,j)=r.b2*(r.A2*t-r.B2/t)/r.e2*s;
   r.Ez(:,1,j)=-r.g0*(r.A2*t+r.B2/t)/r.e2*s;
   t=exp(ki*z(j)*r.b3);
   v=(r.A3.*t+r.B3./t).*r.Cm;
   r.Hy(c,1,j)=v*real(u).*e;
   r.Ex(c,1,j)=((r.b3.*(r.A3.*t-r.B3./t).*r.Cm)*real(u).*e)/r.e3;
   r.Ez(c,1,j)=((r.g3.*v)*imag(u).*e)*(-i/r.e3);
   if isempty(statusbar(j/w(3),f))
      return;
   end
end
%
%Region 4:   Output
%
% Hy = T2*exp(i*k0*(g0*x + b4*z))
%    + sum(Fn*exp(i*k0*(gn*x + bn*(z-h))))
%
n4=sqrt(r.e4);
if O
   [gn,Fn]=fourierCtm(r.T2,r.g0,r.T3.*r.Cm,r.g3,r.k0,r.wx,r.px,r.P);
   u=exp(ki*gn'*(x+r.wx/2));
else
   [gn,Fn]=rayleighCtm(r.T2,r.g0,r.T3.*r.Cm,r.g3,r.k0,r.wx,r.px,r.P);
   u=exp(ki*gn'*x);
end
r.gt=gn;
t=abs(gn-r.g0);
t=find(t == min(t));
r.Ft=real(n1/n4*Fn.*conj(Fn));
r.Ft(t)=real(n1/n4*(Fn(t)+r.T2)*conj(Fn(t)+r.T2));
bn=zSpeed(r.e4,gn);
r.I(:,:,1+b:w(3))=n1/n4;
for j=1+b:w(3)
   t=exp(ki*(z(j)-r.h)*r.b4);
   v=exp(ki*(z(j)-r.h)*bn);
   r.Hy(:,1,j)=r.T2*t*s + (Fn.*v)*u;
   r.Ex(:,1,j)=(r.b4*r.T2*t*s + (Fn.*bn.*v)*u)/r.e4;
   r.Ez(:,1,j)=(-r.g0*r.T2*t*s - (Fn.*gn.*v)*u)/r.e4;
   if isempty(statusbar(j/w(3),f))
      return;
   end
end
%
%Intensity & normalization for 1W/m^2 input
%
% I = 1/2*real(H*conj(H)*sqrt(u0/e0*er))
% s = Hi(I = 1W/m^2)
%
e0=8.85418782e-12; % [As/Vm]
u0=1.25663706e-6;  % [Vs/Am]
s=sqrt(2*n1*sqrt(e0/u0));
r.I=single(real(r.I.*r.Hy.*conj(r.Hy)));
r.Hy=single(r.Hy*s);
s=sqrt(2*n1*sqrt(u0/e0));
r.Ex=single(r.Ex*s);
r.Ez=single(r.Ez*s);
delete(f);
