%r=slitSpectrumTM(r, P, O)
%-------------------------
%
%Computes the spatial spectrum through an array of slits
%in a metal layer. See 'slitTransmissionTM' for details
%on the sample geometry.
%
%The incident plane wave has an intensity of 1W/m^2.
%
%Input:
% r      Structure (see 'slitTransmissionTM')
% P      Compute up to |kx|=P*k {all}
% O      Chooses the plane wave expansion method
%        {'Rayleigh'} or 'Fourier'
%
%Output:
% r      Structure completed with
%  .P       Used limit for kx
%  .R       Spatial spectrum in reflection
%  .gr      kx/k0 and kz/k0 in reflection
%  .br
%  .T       Spatial spectrum in transmission
%  .gt      kx/k0 and kz/k0 in transmission
%  .bt
%
function r=slitSpectrumTM(r, P, O)
if ~nargin | isempty(r) | ~isstruct(r)
   help slitSpectrumTM
   return;
end
if nargin < 2
   P=[];
end
if nargin < 3
   O='';
end
switch lower(O)
case 'fourier'
   O=1;
otherwise
   O=0;
end
if ~isfield(r,'A3')
   r=slitTransmissionTM(r);
end
r.P=abs(P);
%
%Reflection
%
n1=sqrt(r.e1);
if O
   [gn,Fn]=fourierCtm(1+r.R2,r.g0,(1+r.R3).*r.Cm,r.g3,r.k0,r.wx,r.px,real(n1)*r.P);
else
   [gn,Fn]=rayleighCtm(1+r.R2,r.g0,(1+r.R3).*r.Cm,r.g3,r.k0,r.wx,r.px,real(n1)*r.P);
end
r.R=Fn;
r.gr=gn;
r.br=zSpeed(r.e1,gn);
%
%Transmission
%
n4=sqrt(r.e4);
if O
   [gn,Fn]=fourierCtm(r.T2,r.g0,r.T3.*r.Cm,r.g3,r.k0,r.wx,r.px,real(n4)*r.P);
else
   [gn,Fn]=rayleighCtm(r.T2,r.g0,r.T3.*r.Cm,r.g3,r.k0,r.wx,r.px,real(n4)*r.P);
end
r.T=Fn;
r.gt=gn;
r.bt=zSpeed(r.e4,gn);
n='slitSpectrum2D';
f=findobj(get(0,'Children'),'flat','Tag',n);
if isempty(f) | ~ishandle(f)
   f.Parent=figure('Color',[1 1 1],'IntegerHandle','off','MenuBar','none','Name','Spectrum','NumberTitle','off','Tag',n,'ToolBar','none');
   a.Parent=axes(f,'DataAspectRatio',[1 1 1],'NextPlot','add','Position',[0 0 1 1],'XLim',[-1.1 1.1],'YLim',[-1.1 1.1],'Visible','off');
else
   a.Parent=get(f,'CurrentAxes');
   delete(get(a.Parent,'Children'));
   figure(f);
end
%
%Normalize to 1W/m^2 input
%
I=1;
G=r.wx/r.px;
R=real(r.R.*conj(r.R).*r.br/n1);
T=real(r.T.*conj(r.T).*r.bt*(n1/n4));
disp(sprintf('The total absorption is %3.1f%%.',100*(G-sum(R)-sum(T))));
%
%Draw the coordinate system
%
a.LineStyle=':';
c=exp((0:100)*pi/50i);
patch(real(c),imag(c),[1 1 1],'EdgeColor',[0.2 0.2 0.2],a);
a.Color=[0.2 0.2 0.2];
for s=0.2:0.2:0.8
   line(s*real(c),s*imag(c),a);
end
s=exp((0:5)*pi/6i);
line([real(s);-real(s)],[imag(s);-imag(s)],a);
a.Color=[0 0 0];
a.LineStyle='-';
a.LineWidth=2;
line([-1 1;-G G],zeros(2),a);
%
%Separately represent the main reflection
%
t=abs(r.gr-r.g0);
t=find(t == min(t));
G=1-G+R(t)-sum(R);
%
%Normalize the diffracted spectrum
%
s=0.5/max([R T]);
if s > 1
   disp(sprintf('The scattering is drawn %3.1freal values.',s));
   R=R*s;
   T=T*s;
end
%
%Draw the spectra
%
a.LineWidth=1;
line([-r.g0/n1;0],[r.b1/n1;0],a);
line(r.g0/n1*[G;R(t)],r.b1/n1*[G;R(t)],a);
drawArrow(r.g0/n1*G,r.b1/n1*G,pi/2-asin(r.g0/n1),a.Parent);
drawArrow(-r.g0/n1/2,r.b1/n1/2,asin(r.g0/n1)-pi/2,a.Parent);
b.Parent=a.Parent;
b.EdgeColor=[0 0 0];
b.LineStyle='-';
fill(r.gr.*R/n1,r.br.*R/n1,[0.5 0.5 0.5],b);
fill(r.gt.*T/n4,-r.bt.*T/n4,[0.5 0.5 0.5],b);


%Draws a little arrow
%
%Input:
% x      position of the head
% y
% p      alignment [rad]
% a      axes handle
%
function drawArrow(x,y,p,a)
l=0.08;
w=0.02;
R=[cos(p) -sin(p);sin(p) cos(p)];
P=[0 -l -l;0 w -w];
P=R*P;
b=[0 0 0];
fill(P(1,:)+x,P(2,:)+y,b,'EdgeColor',b,'Parent',a);
