%r=spFieldTM(r, w, h, N, P, O)
%-----------------------------
%
%Computes the electromagnetic field through an array of
%slits in a metal layer. See 'slitTransmissionTM' for a
%description of the model geometry.
%
%The incident plane wave has an intensity of 1W/m^2.
%
%Input:
% r      Structure (see 'slitTransmissionTM')
% w      Image width in pixels {256}
% h      Image height in pixels {256}
% N      Number of periods drawn {1}
% P      Compute up to |kx|=P*k {all}
% O      Chooses the plane wave expansion method
%        'Fourier', 'Rayleigh' or {'Plasmon'}
%
%Output:
% r      Structure completed with
%  .I       Intensity [W/m^2]
%  .P       Used limit for kx
%  .S       Drawing dimensions [m]
%  .W       Image dimensions [pixel]
%  .x       Computed points [m]
%  .y
%  .z
%  .Ex      Electric field [V/m]
%  .Ez
%  .Hy      Magnetic field [A/m]
%
function r=spFieldTM(r, w, h, N, P, O)
if ~nargin | isempty(r) | ~isstruct(r)
   help spFieldTM
   return;
end
if nargin < 2 | isempty(w)
   w=256;
end
if nargin < 3 | isempty(h)
   h=256;
end
if nargin < 4 | isempty(N)
   N=1;
end
if nargin < 5
   P=[];
end
if nargin < 6
   O='';
end
switch lower(O)
case 'fourier'
   O=0;
   fprintf('Fourier expansion.\n');
case 'rayleigh'
   O=1;
   fprintf('Rayleigh expansion.\n');
otherwise
   O=2;
   fprintf('Rayleigh expansion with surface plasmons.\n');
end
f=statusbar('Plasmon field ...');
if ~isfield(r,'A3')
   r=slitTransmissionTM(r,P);
end
p=N*r.px/(w-1);
x=((1:w)-w/2)*p;
z=((1:h)-0.3*h)*p+r.h/2;
ki=i*r.k0;
%
%Field allocations
%
r.I=zeros(h,w);
r.P=abs(P);
r.S=[1 0 h/w]*N*r.px;
r.W=[w 1 h];
r.x=x;
r.y=0;
r.z=z;
r.Ex=r.I;
r.Ez=r.I;
r.Hy=r.I;
%
%Regions and slit positions
%
% Metal layer at z(a)...z(b)
% Holes at       x(c)
% Local x        d
% Hole phasors   e
%
a=find(z >= 0);
a=a(1);
b=find(z <= r.h);
b=b(length(b));
c=[];
d=[];
e=[];
for j=-ceil(N):ceil(N)
   t=find(abs(x+j*r.px) < r.wx/2);
   c=[c t];
   d=[d x(t)+r.wx/2+j*r.px];
   e=[e repmat(exp(-j*ki*r.g0*r.px),size(t))];
end
%
%Refraction indices and fourier coefficients
%
n1=sqrt(r.e1);
n2=sqrt(r.e2);
n3=sqrt(r.e3);
n4=sqrt(r.e4);
if O
   [gn,F1]=rayleighCtm(1+r.R2,r.g0,(1+r.R3).*r.Cm,r.g3,r.k0,r.wx,r.px,real(n1)*r.P);
   [gn,F4]=rayleighCtm(r.T2,r.g0,r.T3.*r.Cm,r.g3,r.k0,r.wx,r.px,real(n4)*r.P);
   [R,T,A,B]=layerRTtm(r.e1,r.e2,r.e4,gn,r.k0,r.h);
else
   [gn,F1]=fourierCtm(1+r.R2,r.g0,(1+r.R3).*r.Cm,r.g3,r.k0,r.wx,r.px,real(n1)*r.P);
   [gn,F4]=fourierCtm(r.T2,r.g0,r.T3.*r.Cm,r.g3,r.k0,r.wx,r.px,real(n4)*r.P);
end
%
%Region 1:   Input
%
% Hy = exp(i*k0*(g0*x + b1*z)) + R2*exp(i*k0*(g0*x - b1*z))
%    + sum(Fn*exp(i*k0*(gn*x - bn*z)))
% Ex = b*sqrt(u0/e0)/er*Hy
% Ez = -g*sqrt(u0/e0)/er*Hy
%
s=exp(ki*r.g0*x);
Fn=F1;
if O
   if O > 1
      u=1-r.wx/r.px;
      m=find(abs(gn) > real(n1));
      n=find(abs(gn) > real(n4));
      Fn(m)=F1(m).*(1+u*R(m));
      Fn(n)=Fn(n)+u*F4(n).*(1-R(n));
   end
   u=exp(ki*gn'*x);
else
   u=exp(ki*gn'*(x+r.wx/2));
end
bn=zSpeed(r.e1,gn);
r.I(1:a-1,:)=1;
for j=1:a-1
   l=ki*z(j)*conj(r.b1);
   t=r.R2*exp(conj(l));
   l=exp(l);
   v=exp(-ki*z(j)*bn);
   r.Hy(j,:)=(l+t)*s + (Fn.*v)*u;
   r.Ex(j,:)=(r.b1*(l-t)*s - (Fn.*bn.*v)*u)/r.e1;
   r.Ez(j,:)=(-r.g0*(l+t)*s - (Fn.*gn.*v)*u)/r.e1;
   if isempty(statusbar(j/h,f))
      return;
   end
end
%
%Region 2:   Layer
%
% Hy = A2*exp(i*k0*(g0*x + b2*z)) + B2*exp(i*k0*(g0*x - b2*z))
%
%Region 3:   Slits
%
% Hy = sum((A3*exp(i*k0*b3*z) + B3*exp(-i*k0*b3*z))*cos(k0*g3*x));
%
r.I(a:b,:)=n1/n2;
r.I(a:b,c)=n1/n3;
u=exp(ki*r.g3'*d);
for j=a:b
   t=exp(ki*z(j)*r.b2);
   r.Hy(j,:)=(r.A2*t+r.B2/t)*s;
   r.Ex(j,:)=r.b2*(r.A2*t-r.B2/t)/r.e2*s;
   r.Ez(j,:)=-r.g0*(r.A2*t+r.B2/t)/r.e2*s;
   t=exp(ki*z(j)*r.b3);
   v=(r.A3.*t+r.B3./t).*r.Cm;
   r.Hy(j,c)=v*real(u).*e;
   r.Ex(j,c)=((r.b3.*(r.A3.*t-r.B3./t).*r.Cm)*real(u).*e)/r.e3;
   r.Ez(j,c)=((r.g3.*v)*imag(u).*e)*(-i/r.e3);
   if isempty(statusbar(j/h,f))
      return;
   end
end
%
%Region 4:   Output
%
% Hy = T2*exp(i*k0*(g0*x + b4*z))
%    + sum(Fn*exp(i*k0*(gn*x + bn*(z-h))))
%
Fn=F4;
if O
   if O > 1
      u=1-r.wx/r.px;
      m=find(abs(gn) > real(n4));
      n=find(abs(gn) > real(n1));
      Fn(m)=F4(m).*(1-u*R(m));
      Fn(n)=Fn(n)+u*F1(n).*T(n);
   end
   u=exp(ki*gn'*x);
else
   u=exp(ki*gn'*(x+r.wx/2));
end
bn=zSpeed(r.e4,gn);
r.I(1+b:h,:)=n1/n4;
for j=1+b:h
   t=exp(ki*(z(j)-r.h)*r.b4);
   v=exp(ki*(z(j)-r.h)*bn);
   r.Hy(j,:)=r.T2*t*s + (Fn.*v)*u;
   r.Ex(j,:)=(r.b4*r.T2*t*s + (Fn.*bn.*v)*u)/r.e4;
   r.Ez(j,:)=(-r.g0*r.T2*t*s - (Fn.*gn.*v)*u)/r.e4;
   if isempty(statusbar(j/h,f))
      return;
   end
end
%
%Intensity & normalization for 1W/m^2 input
%
% I = 1/2*real(H*conj(H)*sqrt(u0/e0*er))
% s = Hi(I = 1W/m^2)
%
e0=8.85418782e-12; % [As/Vm]
u0=1.25663706e-6;  % [Vs/Am]
t=[2 3 1];
s=sqrt(2*n1*sqrt(e0/u0));
r.I=permute(single(real(r.I.*r.Hy.*conj(r.Hy))),t);
r.Hy=permute(single(r.Hy*s),t);
s=sqrt(2*n1*sqrt(u0/e0));
r.Ex=permute(single(r.Ex*s),t);
r.Ez=permute(single(r.Ez*s),t);
delete(f);
