%f=contour3D(r, N)
%-----------------
%
%Draws the intensity profile at the xy, xz and yz cross-sections.
%
%Input:
% r      Structure (see 'holeField')
% N      Number of xy-slices {1}
%
%Output:
% f     Figure handle
%
function f=contour3D(r, N)
if nargin < 2 | isempty(N)
   N=1;
end
u=find(r.x >= 0);
v=find(r.y >= 0);
w=find(r.z >= r.h);
%
%Scale 1. quadrant
%
x=r.x(u)*1e9;
y=r.y(v)*1e9;
z=r.z(w)*1e9;
I=double(r.I(u,v,w));
L=max(I(:))*exp(-0.5:-0.5:-6);
[u,v,w]=size(I);
%
%Create figure
%
t.FontSize=14;
f=figure('Name','Intensity profile','NumberTitle','off');
a.Parent=axes('Parent',f,'DataAspectRatio',[1 1 1],t,'Position',[0.12 0.05 0.85 0.93],'XLim',[x(1) x(u)],'YLim',[y(1) y(v)],'ZLim',[z(1) z(w)]);
t.HorizontalAlignment='center';
t.VerticalAlignment='middle';
t.Units='Normalized';
text(0.9,-0.02,'x [nm]',t);
text(0.1,-0.02,'y [nm]',t);
text(-0.1,0.98,'z [nm]',t);
view(125,35-5*N);
camlight;
%
%xy sections
%
P=zeros(u,v);
for h=1:ceil(w/N):w
   P(:,:)=I(:,:,h);
   surface(repmat(x',1,v),repmat(y,u,1),repmat(z(h),u,v),P,'EdgeColor','none',a);
   c=contourc(y,x,P,L);
   while ~isempty(c)
      m=c(2,1);
      n=2:m+1;
      line(c(2,n),c(1,n),repmat(z(h),1,m),lineStyle(c(1,1),L),a);
      c=c(:,2+m:size(c,2));
   end
end
%
%xz section
%
P=zeros(u,w);
P(:,:)=I(:,1,:);
surface(repmat(x',1,w),repmat(y(1),u,w),repmat(z,u,1),P,'EdgeColor','none',a);
c=contourc(z,x,P,L);
while ~isempty(c)
   m=c(2,1);
   n=2:m+1;
   line(c(2,n),repmat(y(1),1,m),c(1,n),lineStyle(c(1,1),L),a);
   c=c(:,2+m:size(c,2));
end
%
%yz section
%
P=zeros(v,w);
P(:,:)=I(1,:,:);
surface(repmat(x(1),v,w),repmat(y',1,w),repmat(z,v,1),P,'EdgeColor','none',a);
c=contourc(z,y,P,L);
while ~isempty(c)
   m=c(2,1);
   n=2:m+1;
   line(repmat(x(1),1,m),c(2,n),c(1,n),lineStyle(c(1,1),L),a);
   c=c(:,2+m:size(c,2));
end
%
%Enforce coordinate axes
%
m=ones(1,N);
h=1:ceil(w/N):w;
line(x([m m 1;m*u m 1]),y([m m 1;m m*v 1]),z([h h 1;h h w]),'Color',[0 0 0],a);


function o=lineStyle(I, L)
m=find(L == I);
c=[1 0.75 0.5];
s={'-','--',':'};
o.LineStyle=s{ceil(m/4)};
o.Color=repmat(c(ceil(m/4)),1,3);
