%r=holeFieldTE(r, s, w, P, O)
%----------------------------
%
%Computes the electromagnetic field through an array of
%holes in a metal layer. See 'holeTransmissionTE' for a
%description of the model geometry.
%
%The incident plane wave has an intensity of 1W/m^2.
%
%Input:
% r      Structure (see 'holeTransmissionTE')
% s      Drawing dimensions [m] {[px py h+3*l]}
%     or
%        Drawing intervals [m] if 3x2 matrix
% w      Image dimensions [pixel] {[100 100 100]}
% P      Compute up to |kx|=P*k {10}
% O      Chooses the plane wave expansion method
%        {'Rayleigh'} or 'Fourier'
%
%Output:
% r      Structure completed with
%  .I       Intensity [W/m^2]
%  .P       Used limit for kx
%  .S       Drawing dimensions [m]
%  .W       Image dimensions [pixel]
%  .x       Computed points [m]
%  .y
%  .z
%  .Ex      Electric field [V/m]
%  .Ey
%  .Ez
%  .Hx      Magnetic field [A/m]
%  .Hy      (not yet implemented)
%  .Hz
%  .gr      kx/k0+i*ky/kz of spectra
%  .Fr      Reflection spectrum [W/m^2]
%  .gt      kx/k0+i*ky/kz of spectra
%  .Ft      Transmission spectrum [W/m^2]
%
function r=holeFieldTE(r, s, w, P, O)
if ~nargin | isempty(r) | ~isstruct(r)
   help holeFieldTE
   return;
end
if nargin < 2 | length(s) ~= 3
   s=[r.px r.py r.h+3*r.l];
end
if nargin < 3 | length(w) ~= 3
   w=[100 100 100];
else
   w=ceil(w);
end
if nargin < 4 | length(P) ~= 1
   P=10;
end
if nargin < 5
   O='';
end
switch lower(O)
case 'fourier'
   O=1;
otherwise
   O=0;
end
f=statusbar('Field computation ...');
if ~isfield(r,'A3')
   r=holeTransmissionTE(r,P);
end
ki=i*r.k0;
if size(s) == [3 2]
   d=s(:,1);
   s=(s(:,2)-s(:,1))';
   t=s./w;
   x=((0.5:w(1)))*t(1)+d(1);
   y=((0.5:w(2)))*t(2)+d(2);
   z=((0.5:w(3)))*t(3)+d(3);
else
   t=s./w;
   x=((1:w(1))-(1+w(1))/2)*t(1);
   y=((1:w(2))-(1+w(2))/2)*t(2);
   z=((1:w(3))-w(3)/3)*t(3) + r.h/2;
end
%
%Field allocations
%
r.I=zeros(w);
r.P=abs(P);
r.S=s;
r.W=w;
r.x=x;
r.y=y;
r.z=z;
r.Ex=r.I;
r.Ey=r.I;
r.Ez=r.I;
%r.Hx=r.I;
%r.Hy=r.I;
%r.Hz=r.I;

%
%Regions and hole positions
%
% Metal layer at z(a)...z(b)
% Holes at       x(c) X y(d)
% Hole phasors   e
% Local x        f
%
a=find(z >= 0);
a=max([1 a(1)]);
b=find(z <= r.h);
if length(b)
   b=b(length(b));
else
   b=0;
end
c=[];
d=[];
e=[];
f=[];
n=ceil(s(1)/r.px);
for j=-n:n
   t=find(abs(x-j*r.px) < r.wx/2);
   c=[c t];
   e=[e repmat(exp(j*ki*r.g0*r.px),size(t))];
   f=[f x(t)+r.wx/2-j*r.px];
end
n=ceil(s(2)/r.py);
for j=-n:n
   t=find(abs(y-j*r.py) < r.wy/2);
   d=[d t];
end
%
%Region 1:   Input
%
% Ex = sum(bx*Fn*exp(i*k0*(gx*x + gy*y - bn*z)))
% Ey = exp(i*k0*(g0*x + b1*z)) + R2*exp(i*k0*(g0*x - b1*z))
%    + sum(by*Fn*exp(i*k0*(gx*x + gy*y - bn*z)))
% Ez = sum(bz*Fn*exp(i*k0*(gx*x + gy*y - bn*z)))
%
s=exp(ki*x'*r.g0);
n1=sqrt(r.e1);
if O
   [gx,gy,Fn]=fourierCte(1+r.R2,r.g0,(1+r.R3).*r.Cm,r.g3,r.k0,r.wx,r.wy,r.px,r.py,real(n1)*r.P);
   u=exp(ki*(x+r.wx/2)'*gx');
else
   [gx,gy,Fn]=rayleighCte(1+r.R2,r.g0,(1+r.R3).*r.Cm,r.g3,r.k0,r.wx,r.wy,r.px,r.py,real(n1)*r.P);
   u=exp(ki*x'*gx');
end
[bx,by,bz]=projectEy(gx,gy,r.e1);
bn=zSpeed(r.e1,gx,gy);
Fx=Fn.*bx;
Fy=Fn.*by;
Fz=Fn.*bz;
%
%Spectrum
%
r.gr=complex(gx,gy);
Fy=Fy+(r.gr == r.g0)*r.R2;
r.Fr=real(Fx.*conj(Fx)+Fy.*conj(Fy)+Fz.*conj(Fz));
r.I(:,:,1:a-1)=1;
for j=1:a-1
   t=s*exp(ki*z(j)*conj(r.b1));
   v=exp(-ki*z(j)*bn);
   for i=1:length(y)
      p=exp(ki*y(i)*gy).*v;
      r.Ex(:,i,j)=u*(Fx.*p);
      r.Ey(:,i,j)=t + u*(Fy.*p);
      r.Ez(:,i,j)=u*(Fz.*p);
   end
   if isempty(statusbar(j/w(3),f))
      return;
   end
end
%
%Region 2:   Layer
%
% Ex = 0
% Ey = A2*exp(i*k0*(g0*x + b2*z)) + B2*exp(i*k0*(g0*x - b2*z))
% Ez = 0
%
%Region 3:   Holes
%
% Ex = 0
% Ey = A3*exp(i*k0*(g3*x + b3*z)) + B3*exp(i*k0*(g3*x - b3*z))
% Ez = 0
%
n2=sqrt(r.e2);
n3=sqrt(r.e3);
r.I(:,:,a:b)=n2/n1;
r.I(c,d,a:b)=n3/n1;
u=exp(ki*r.g3'*f);
for j=a:b
   t=exp(ki*z(j)*r.b2);
   r.Ey(:,:,j)=repmat(s*(r.A2*t+r.B2/t),size(y));
   t=exp(ki*z(j)*r.b3);
   r.Ey(c,d,j)=repmat((((r.A3.*t+r.B3./t).*r.Cm)*imag(u).*e).',size(d));
   if isempty(statusbar(j/w(3),f))
      return;
   end
end
%
%Region 4:   Output
%
% Ex = sum(bx*Fn*exp(i*k0*(gx*x + gy*y + bn*(z-h))))
% Ey = T2*exp(i*k0*(g0*x + b4*z))
%    + sum(by*Fn*exp(i*k0*(gx*x + gy*y + bn*(z-h))))
% Ez = sum(bz*Fn*exp(i*k0*(gx*x + gy*y + bn*(z-h))))
%
n4=sqrt(r.e4);
if O
   [gx,gy,Fn]=fourierCte(r.T2,r.g0,r.T3.*r.Cm,r.g3,r.k0,r.wx,r.wy,r.px,r.py,real(n4)*r.P);
   u=exp(ki*(x+r.wx/2)'*gx');
else
   [gx,gy,Fn]=rayleighCte(r.T2,r.g0,r.T3.*r.Cm,r.g3,r.k0,r.wx,r.wy,r.px,r.py,real(n4)*r.P);
   u=exp(ki*x'*gx');
end
[bx,by,bz]=projectEy(gx,gy,r.e4);
bn=zSpeed(r.e4,gx,gy);
Fx=Fn.*bx;
Fy=Fn.*by;
Fz=Fn.*bz;
   %
   %Spectrum
   %
   r.gt=complex(gx,gy);
   Fy=Fy+(r.gt == r.g0)*r.T2;
   r.Ft=real(n4/n1.*(Fx.*conj(Fx)+Fy.*conj(Fy)+Fz.*conj(Fz)));
r.I(:,:,1+b:w(3))=n4/n1;
for j=1+b:w(3)
   v=exp(ki*(z(j)-r.h)*bn);
   for i=1:length(y)
      p=exp(ki*y(i)*gy).*v;
      r.Ex(:,i,j)=u*(Fx.*p);
      r.Ey(:,i,j)=u*(Fy.*p);
      r.Ez(:,i,j)=u*(Fz.*p);
   end
   if isempty(statusbar(j/w(3),f))
      return;
   end
end
%
%Intensity & normalization for 1W/m^2 input
%
% I = 1/2*(E*E')*sqrt(e0/u0)*real(sqrt(er))
% s = Ei(I = 1W/m^2)
%
e0=8.85418782e-12; % [As/Vm]
u0=1.25663706e-6;  % [Vs/Am]
s=sqrt(2/n1*sqrt(u0/e0));
r.I=single(real(r.I.*(r.Ex.*conj(r.Ex)+r.Ey.*conj(r.Ey)+r.Ez.*conj(r.Ez))));
r.Ex=single(r.Ex*s);
r.Ey=single(r.Ey*s);
r.Ez=single(r.Ez*s);
delete(f);
