%r=holeTransmissionTE(r, P)
%--------------------------
%
%Computes the electromagnetic field produced by a single hole
%as shown below when lit with a plane TE polarised wave with
%the xz plane as incidence plane.
%
%             wx(wy)
%     1      -----      1
%               y
%------------ --o-- ------------       -      ---> x
%                                    
%     2        3        2          h 
%                                    
%-------------------------------       -
%
%               4
%
%-------------------------------
%
%               
%               
%               v
%               z
%
%
%The incident light impinges through the dielectricum 1 onto
%the metal layer 2 and the hole 3. A part is diffracted back
%into the region 1, another part is transmitted through the
%hole into the substrate 4.
%
%The incident light has an electric field Ey along the y-axis.
%Hence, its magnetic field H=(Hx,0,Hz) lies in the xz-plane.
%At the boundaries, the surfacial components of the magnetic
%and of the electric field have to be continuos.
%
%Input:
% r      Structure
%  .i       Incidence angle []
%              0 = z-axis
%             90 = x-axis
%  .l       Wavelength in free space [m]
%  .h       Thickness of the metal layer [m]
%  .px      Distance between the holes [m]
%  .py
%  .wx      Width of the holes [m]
%  .wy
%  .e1      Relative dielectric constants
%  .e2
%  .e3
%  .e4
%
% P      Compute up to |kx|=P*k {10}
%
%Output:
% r      Structure completed with
%  .k0      Wavevector in free space [rad/m]
%  .g0      kx/k0 of the incident wave
%  .g3      kx/k0 of the TE hole modes
%  .b1      Propagation constants kz/k0
%  .b2
%  .b3
%  .b4
%  .Cm      Coupling coefficients to the hole modes
%  .A2      Coefficients in region 2/3 of the forward propagating wave
%  .A3
%  .B2      Coefficients in region 2/3 of the backward propagating wave
%  .B3
%  .R2      Reflection coefficients at the 1 -> 2/3 -> 4 layers
%  .R3
%  .T2      Transmission coefficients through the 1 -> 2/3 -> 4 layers
%  .T3
%
function r=holeTransmissionTE(r, P)
if nargin < 2 | isempty(P)
   P=10;
else
   P=max(1,P);
end
%
%Wavevector and propagation constants
%
s=r.l/r.wx/2;
r.k0=2*pi/r.l;
r.g0=sqrt(r.e1)*sin(r.i*pi/180);
r.g3=s*(1:ceil(P/s));
r.b1=zSpeed(r.e1,r.g0);
r.b2=zSpeed(r.e2,r.g0);
r.b3=zSpeed(r.e3,r.g3);
r.b4=zSpeed(r.e4,r.g0);
%
%Fresnel reflection and transmission through the metal layer
%
[R,T,A,B]=layerRTte(r.e1,r.e2,r.e4,r.g0,r.k0,r.h);
r.R2=R;
r.T2=T;
r.A2=A;
r.B2=B;
%
%Coupling coefficients to the TE hole modes
%
r.Cm=holeCte(r.g0,r.g3,r.k0,r.wx);
%
%Fresnel reflection and transmission through the hole
%
[R,T,A,B]=layerRTte(r.e1,r.e3,r.e4,r.g3,r.k0,r.h);
r.R3=R;
r.T3=T;
r.A3=A;
r.B3=B;
