%s=lensTransmission(s, P)
%------------------------
%
%Input:
% s      Data structure
%  .e       Relative dielectric constant
%  .l       Wavelength in free space [m]
%  .d       Lens diameter [m]
%  .f       Back focal length [m]
%  .p       Period of lens array [m]
% P      Compute up to |k|=P*k0 {10}
%
%Output:
% s      Modified data structure
%  .P       Compute up to |k|=P*k0
%  .k0      Wavevector [rad/m]
%  .xp      Pupil coordinates [m]
%  .yp
%  .rp
%  .tx      Transmisson coefficients [V/m]
%  .ty
%  .tz
%  .tp      Phase and apodisation [cis]
%
function s=lensTransmission(s, P)
if nargin < 2 | isempty(P)
   P=10;
end
s.P=P;
s.k0=2*pi/s.l;
%
%Sampling
%
t=s.l/2/P;
n=2*ceil(s.d/2/t)+1;
t=s.d/n;
x=repmat(((0.5:n)-n/2)'*t,1,n);
y=repmat(((0.5:n)-n/2)*t,n,1);
%
%Pupil region
%
R=x.^2+y.^2;
n=find(R < s.d^2/4);
%
%Pupil coordinates (x/y)
%
R=R(n);
s.xp=x(n);
s.yp=y(n);
s.rp=sqrt(R);
%
%Intermediate constants
%
fr=sqrt(s.f^2+R);
cT=s.f./fr;
sT=s.rp./fr;
ct=s.rp == 0;
cP=(s.xp+ct)./(s.rp+ct);
sP=s.yp./(s.rp+ct);
ct=cT-1;
%
%Transmission coefficients
%
s.tx=[ct.*cP.^2+1 ct.*cP.*sP];
s.ty=[s.tx(:,2) ct.*sP.^2+1];
s.tz=[sT.*cP sT.*sP];
%
%Phase and apodisation
%
s.tp=sqrt(cT).*exp(i*s.k0*real(sqrt(s.e))*(fr-s.f));
