%[gx,gy,Fn]=rayleighCte(C0,g0,Cm,gm,k0,wx,wy,px,py,P)
%----------------------------------------------------
%
%Computes the field difference at the hole boundary and
%returns the coefficients of the Rayleigh expansions in
%x and y.
%
%Input:
% C0     Coefficient at the metal layer
% g0     kx/k0 of the incident wave
% Cm     Modal coefficients at the hole
% gm     kx/k0 of the TE hole modes
% k0     Wavevector in free space [rad/m]
% wx     Width of the holes [m]
% wy
% px     Distance between the holes [m]
% py
% P      Compute up to |kx|=P*k0 {all}
%
%Output:
% gx     kx/k0 of the diffracted waves
% gy
% Fn     Rayleigh coefficients
%
function [gx,gy,Fn]=rayleighCte(C0,g0,Cm,gm,k0,wx,wy,px,py,P)
%
%Along the x axis
%
t=max(50,5*length(gm));
s=wx/t/2;
x=s*(-t:t);
t=2*t*px/wx;
dF=Cm*sin(k0*gm'*(x+wx/2))-C0*exp(i*k0*g0*x);
%
%Avoid aliasing (Shannon)
%
P=min([P pi/k0/s]);
%
%Rayleigh expansion
%
% gx = g0 + n*px/l    for all |gx| < P
%
s=2*pi/k0/px;
gx=g0+(-floor((P+g0)/s):floor((P-g0)/s))*s;
Fx=dF*exp(-i*k0*x'*gx)/t;
%
%Along the y axis
%
s=2*pi/k0/py;
t=floor(P/s);
gy=(-t:t)*s;
Fy=wy/py*sinc(wy/py*(-t:t));
%
%Combine the results
%
gx=gx';
Fn=Fx.'*Fy;
t=size(gx);
gx=repmat(gx,size(gy));
gy=repmat(gy,t);
t=find(gx.^2+gy.^2 < P^2);
gx=gx(t);
gy=gy(t);
Fn=Fn(t);
