%[e2,h2]=globalReh(e1,e3, k0, Rm,tm, s)
%--------------------------------------
%
%Fitted parameters from a Kretschmann setup.
%See 'localReh' for details.
%
%Input:
% e1     Dielectric constant of the sample holder
% e3     Dielectric constant behind the film
% k0     Wavevector in free space [rad/m]
% Rm     Measured reflection efficiencies
% tm     Incidence angles []
% s      Window title
%
%Output:
% e2     Dielectric constant of the metal film
% h2     Thickness of the metal film [m]
%
function [e2,h2]=globalReh(e1,e3, k0, Rm,tm, s)
m=[1+floor((0:4)/4*(length(Rm)-1)) find(Rm == min(Rm)) find(Rm == max(Rm))];
gm=sqrt(e1)*sin(tm(:)*pi/180);
Rm=Rm(:);
%
%Retrieve the angle and reflectance of the flattest curve part
%
g0=abs(derive(Rm,1,gm));
g0=find(g0 == min(g0));
g0=[max(1,g0-1):min(g0+1,length(Rm))];
%
%Compensate the damping due to the sample holder
%
% The sample holder is assumed to reduce the reflection
% efficiency by a constant factor (about 0.92 for a glass
% cylinder due to two glass-air interfaces for example).
%
Rm=Rm/mean(Rm(g0));
g0=gm(g0);
%
%Look for the global error minimum
%
h2=10e-9:10e-9:100e-9;
er=-1.5:-0.5:-20;
ei=0.1:0.1:5;
e2=complex(repmat(er',size(ei)),repmat(ei,size(er')));
e=zeros(length(h2),length(er),length(ei));
for a=1:length(h2)
   for b=1:length(e2(:))
      e(a,b)=localReh([e2(b);h2(a)],e1,e3,k0,g0,gm(m),Rm(m));
   end
end
[a,b]=find(e == min(e(:)));
%
%Refine the global minimum looking for the local minimum
%
[m,e]=fminsearch(@localReh,[e2(b);h2(a)],optimset('Display','off','TolX',1e-4),e1,e3,k0,g0,gm,Rm);
e2=m(1);
h2=m(2);
%
%Display results and diagnostics
%
tt=min(tm):0.01:max(tm);
gm=sqrt(e1)*sin(tt*pi/180);
Rt=layerRTtm(e1,e2,e3,gm,k0,h2);
R0=layerRTtm(e1,e2,e3,g0,k0,h2);
figure('Name',s,'NumberTitle','off','Toolbar','none');
plot(tm,Rm,'ro',tt,real(Rt.*conj(Rt)/mean(R0.*conj(R0))),'b-');
fprintf('Found:\n\te2 = %g %+gi\n\th2 = %gnm\n\nMean error is %g%%.\n',[real(e2) imag(e2) 1e9*h2 100*sqrt(e/length(Rm))]);
set(gca,'Box','off','FontSize',14,'Position',[0.08 0.08 0.8 0.8],'XLim',[min(tm) max(tm)]);
text(1.05,-0.04,'ti','FontSize',14,'HorizontalAlignment','left','VerticalAlignment','middle','Units','Normalized');
text(-0.02,1.06,'R','FontSize',14,'HorizontalAlignment','center','VerticalAlignment','baseline','Units','Normalized');
grid on;
%
%Special
%
h=get(gca,'Children');
set(gca,'XLim',[41.5 80],'YLim',[0 1.2],'YTick',[0:0.2:1.2],'YTickLabel',['0.0';'0.2';'0.4';'0.6';'0.8';'1.0';'1.2']);
set(h(4),'MarkerEdgeColor','black','MarkerFaceColor','black','MarkerSize',3);
set(h,'Color','black');
text(62,0.6,'eh','FontSize',14);
